@@ -37,28 +37,28 @@ retistruct.cli <- function(dataset, cpu.time.limit=Inf, outputdir=NA,
 }
 
 ##' This function processes a \code{dataset}, saving the
-##' reconstruction data and matlab export data to the \code{dataset}
+##' reconstruction data and MATLAB export data to the \code{dataset}
 ##' directory and printing figures to \code{outputdir}.
 ##'
 ##' @title Process a dataset, saving results to disk
 ##' @param dataset Path to dataset to process 
 ##' @param outputdir Directory in which to save any figures
 ##' @param device String representing device to print figures to
-##' @param titrate If \code{TRUE} add output of
-##' \code{\link{titrate.reconstructedOutline}}  to object saved.
 ##' @author David Sterratt
 ##' @export
-retistruct.cli.process <- function(dataset, outputdir=NA, device="pdf",
-                                   titrate=FALSE) {
+retistruct.cli.process <- function(dataset, outputdir=NA, device="pdf"
+                                   ## titrate=FALSE   ## FIXME: Issue #25: Titration
+                                   ) {
   ## Processing
   warn.opt <- getOption("warn")
   options(warn=1)
   r <- retistruct.read.dataset(dataset)
   r <- retistruct.read.markup(r)
   r <- retistruct.reconstruct(r)
-  if (titrate) {
-    r$titration <- titrate.reconstructedOutline(r)
-  }
+  ## FIXME: Issue #25: Titration
+  ## if (titrate) {
+  ##   r$titration <- titrate.reconstructedOutline(r)
+  ## }
   ## Output
   retistruct.save.recdata(r)
   
